//  This file is really -*-C++-*-
//  File:                     RedWhiteBlue.h
//  Created by:               David Casperson <casper@ugrad.unbc.ca>
//  Created:                  Fri Sep 17 09:48:47 1999
//
//  Description:              Interface to a RedWhiteBlue class for use in 
//                            Dijkstra's Dutch Flag problem.  All
//                            concrete class details are hidden using
//                            the p_impl idiom.

#if !defined(RedWhiteBlue_INCLUDED)

#include <vector>
class RedWhiteBlue_Implementation ;

class RedWhiteBlue
{
public:
    // constructors, destructors, and assignment operators
    RedWhiteBlue(const RedWhiteBlue&) ;
    RedWhiteBlue& operator= (const RedWhiteBlue&) ;
    ~RedWhiteBlue() ;
    // Note that there is no default constructor!

    // You can (quickly) swap this RWB object with another
    void swap(RedWhiteBlue&) ;

    // If you really really want an object of a particular colour
    // you can make a copy of one of the following static member variables.
    static const RedWhiteBlue red ;
    static const RedWhiteBlue white ;
    static const RedWhiteBlue blue ;

    // To create a random vector of RedWhiteBlue objects, call the
    // following static member function.  The parameters r,w,b are the
    // number of red, white, and blue objects respectively to randomly
    // insert in the vector.
    static std::vector<RedWhiteBlue> makeRandomRWBVector(int r, int w, int b) ;

    // Queries to determine the color of an object.
    bool isRed  () const ;
    bool isWhite() const ;
    bool isBlue () const ;

private:    // hands off! :)
    RedWhiteBlue(RedWhiteBlue_Implementation*) ;
    RedWhiteBlue_Implementation * p_impl ;
} ;


#define RedWhiteBlue_INCLUDED
#endif  // !defined(RedWhiteBlue_INCLUDED)
